@@ -3,12 +3,10 @@ from datetime import datetime, timedelta
 from itertools import groupby
 from urllib.parse import quote as urlquote
 
-import click
-#import requests  # we use lazy imports to improve load time for local commands
-#from bs4 import BeautifulSoup
-#from bs4.element import NavigableString
+# import requests  # we use lazy imports to improve load time for local commands
+# from bs4 import BeautifulSoup
+# from bs4.element import NavigableString
 
-from kks.errors import AuthError
 from kks.util.h2t import HTML2Text
 
 CONTEST_ID_BY_GROUP = {
@@ -17,13 +15,14 @@ CONTEST_ID_BY_GROUP = {
 }
 
 
-class LinkTypes:
-    SETTINGS = 'Settings'
-    SUMMARY = 'Summary'
-    SUBMISSIONS = 'Submissions'
-    USER_STANDINGS = 'User standings'
-    SUBMIT_CLAR = 'Submit clar'
-    CLARS = 'Clars'
+class Links:
+    WEB_CLIENT_ROOT = 'https://caos.ejudge.ru/ej/client'
+    SETTINGS = f'{WEB_CLIENT_ROOT}/view-settings/S__SID__'
+    SUMMARY = f'{WEB_CLIENT_ROOT}/view-problem-summary/S__SID__'
+    SUBMISSIONS = f'{WEB_CLIENT_ROOT}/view-submissions/S__SID__'
+    USER_STANDINGS = f'{WEB_CLIENT_ROOT}/standings/S__SID__'
+    SUBMIT_CLAR = f'{WEB_CLIENT_ROOT}/view-clar-submit/S__SID__'
+    CLARS = f'{WEB_CLIENT_ROOT}/view-clars/S__SID__'
 
 
 class Status:
@@ -44,15 +43,17 @@ class AuthData:
         self.password = password
 
 
-class Problem:
+class BaseProblem:
     def __init__(self, short_name, name, href, status, tests_passed, score):
         self.short_name = short_name
         self.name = name
-        self.href = href  # NOTE contains SID -> quickly becomes outdated
+        self.href = href
         self.status = status
         self.tests_passed = tests_passed
         self.score = score
 
+
+class Problem(BaseProblem):
     def color(self):
         return 'green' if self.status == Status.OK \
             else 'green' if self.status == Status.REVIEW \
@@ -64,12 +65,18 @@ class Problem:
     def bold(self):
         return self.status == Status.OK
 
+    def get_full(self, session):
+        # TODO use API? (see #68)
+        page = session.get(self.href)
+        return FullProblem(self, page)
+
 
 class Submission:
     def __init__(self, row):
         cells = row.find_all('td')
         self.id = int(cells[0].text)
         self.problem = cells[3].text
+        self.compiler = cells[4].text
         self.status = cells[5].text
         self.source = cells[8].find('a')['href'].replace('view-source', 'download-run')
         report_link = cells[9].find('a')
@@ -86,6 +93,17 @@ class Submission:
             return 'Partial'
         return self.status
 
+    def suffix(self):
+        if self.compiler.startswith('gas'):
+            return '.S'
+        if '++' in self.compiler:
+            return '.cpp'
+        if self.compiler.startswith('make'):
+            return '.tar'
+        if self.compiler.startswith('gcc') or self.compiler.startswith('clang'):
+            return '.c'
+        return ''
+
 
 class Report:
     def __init__(self, comments, tests):
@@ -102,18 +120,19 @@ class Report:
         else:
             self.lines = []
 
-        has_failed_tests = False
+        failed_tests = []
 
         for test in tests:
-            t_id, status, *_ = map(lambda cell: cell.text, test.find_all('td'))
-            if status != Status.OK:
-                if not has_failed_tests:
-                    if comments:
-                        self.lines.append('\n')
-                    self.lines.append(f'Total tests: {len(tests)}\n')
-                    self.lines.append('Failed tests:\n')
-                    has_failed_tests = True
-                self.lines.append(f'{t_id} - {status}\n')
+            test_num, status, *_ = test.find_all('td')
+            if status.text != Status.OK:
+                failed_tests.append(f'{test_num.text} - {status.text}\n')
+
+        if failed_tests:
+            if comments:
+                self.lines.append('\n')
+            self.lines.append(f'Total tests: {len(tests)}\n')
+            self.lines.append('Failed tests:\n')
+            self.lines += failed_tests
 
     def as_comment(self):
         sep_lines = 3
@@ -121,9 +140,8 @@ class Report:
 
 
 class TaskScore:
-    def __init__(self, contest, task_name, score, status):
+    def __init__(self, contest, score, status):
         self.contest = contest
-        self.task_name = task_name
         self.score = score
         self.status = status
 
@@ -131,39 +149,36 @@ class TaskScore:
         return 'green' if self.status == Status.REVIEW \
             else 'green' if self.status == Status.OK \
             else 'bright_yellow' if self.status == Status.TESTING \
+            else 'yellow' if self.status == Status.REJECTED \
             else 'red' if self.status == Status.PARTIAL \
             else 'white'
+        # in standings CHECK has the same style as TESTING
 
     def bold(self):
-        return self.status == Status.OK
+        # TESTING is bold for more contrast with REJECTED
+        return self.status in [Status.OK, Status.TESTING]
 
     def table_score(self):
-        if self.status == Status.TESTING and self.score is None:
+        if self.status in [Status.TESTING, Status.REJECTED] and self.score is None:
             return '??'
         return self.score
 
 
-class Standings:
-    def __init__(self, task_names, rows):
-        self.task_names = task_names
-        self.rows = rows
-
-        self.contests = [contest for contest, _ in groupby(task_names, extract_contest_name)]
-
-        self.tasks_by_contest = {
-            contest: list(tasks)
-            for contest, tasks in groupby(task_names, extract_contest_name)
-        }
+class TaskInfo:
+    def __init__(self, name, contest):
+        self.name = name
+        self.contest = contest
 
 
 class StandingsRow:
-    def __init__(self, place, user, tasks, solved, score, is_self):
+    def __init__(self, place, user, tasks, solved, score, is_self, contest_id=None):
         self.place = place
         self.user = user
         self.tasks = tasks
         self.solved = solved
         self.score = score
         self.is_self = is_self
+        self.contest_id = contest_id
 
     def color(self):
         return 'white'
@@ -172,6 +187,22 @@ class StandingsRow:
         return self.is_self
 
 
+class Standings:
+    def __init__(self, tasks, rows):
+        self.tasks = tasks
+        self.rows = rows
+
+        self.contests = [
+            contest
+            for contest, tasks in groupby(self.tasks, lambda task: task.contest)
+        ]
+
+        self.tasks_by_contest = {
+            contest: list(tasks)
+            for contest, tasks in groupby(self.tasks, lambda task: task.contest)
+        }
+
+
 class ProblemInfo:
     """Subset of task info table used for max score estimation"""
 
@@ -181,33 +212,52 @@ class ProblemInfo:
         self.current_penalty = current_penalty
 
 
-class Statement:
-
+class FullProblem(BaseProblem):
     keep_info = ['Time limit:', 'Real time limit:', 'Memory limit:']
 
-    def __init__(self, page):
+    def __init__(self, problem, page):
+        super().__init__(**problem.__dict__)
         from bs4 import BeautifulSoup
-        from bs4.element import NavigableString
-
-        self.input_data = None
-        self.output_data = None
-        self._html = None
-        self.url = page.url
 
         soup = BeautifulSoup(page.content, 'html.parser')
         task_area = soup.find('div', {'id': 'probNavTaskArea'})
 
-        input_title = task_area.find('h4', text='Input')
+        self.input_data, self.output_data = self.parse_sample(task_area)
+        self._html = self.parse_statement(task_area)
+        self._suffix = self.guess_suffix(task_area)
+        self.url = page.url
+
+    def suffix(self):
+        if self._suffix is not None:
+            return self._suffix
+        pref = self.name.split('/', 1)[0]
+        if pref == 'asm':
+            return '.S'
+        return '.c'
+
+    @staticmethod
+    def parse_sample(html):
+        input_data, output_data = None, None
+
+        input_title = html.find('h4', text='Input')
         if input_title is not None:
-            self.input_data = input_title.find_next('pre').text
+            input_data = input_title.find_next('pre').text
 
-        output_title = task_area.find('h4', text='Output')
+        output_title = html.find('h4', text='Output')
         if output_title is not None:
-            self.output_data = output_title.find_next('pre').text
+            output_data = output_title.find_next('pre').text
 
-        problem_info = task_area.find('table', {'class': 'line-table-wb'})
+        return input_data, output_data 
+
+    @staticmethod
+    def parse_statement(html):
+        from bs4 import BeautifulSoup
+        from bs4.element import NavigableString
+
+        soup = BeautifulSoup()
+        problem_info = html.find('table', {'class': 'line-table-wb'})
         if problem_info is None:
-            return
+            return None
         next_block = problem_info.find_next('h3', text='Submit a solution')
         if next_block is None:
             next_block = problem_info.find_next('h2')
@@ -218,12 +268,15 @@ class Statement:
         info_avail = False
         for row in problem_info.find_all('tr'):
             key, value = row.find_all('td')
-            if key.text in Statement.keep_info:
+            if key.text in FullProblem.keep_info:
                 info_avail = True
                 info.append(copy(row))
+                info.append('\n')
 
         if info_avail:
+            statement.append('\n')
             statement.append(info)
+            statement.append('\n')
 
         statement_avail = False
         curr = problem_info.next_sibling
@@ -235,22 +288,56 @@ class Statement:
             statement.append(copy(curr))
             curr = curr.next_sibling
 
-        if statement_avail:
-            html = soup.new_tag('html')
-            head = soup.new_tag('head')
-            head.append(soup.new_tag('meta', charset='utf-8'))
-            html.append(head)
-            html.append(statement)
-            self._html = html
+        if not statement_avail:
+            return None
+
+        html = soup.new_tag('html')
+        head = soup.new_tag('head')
+        head.append(soup.new_tag('meta', charset='utf-8'))
+        html.append(head)
+        html.append('\n')
+        html.append(statement)
+        return html
+
+    @staticmethod
+    def guess_suffix(html):
+
+        def get_suf(lang_id):
+            # NOTE compiler ids may change
+            lang_id = int(lang_id)
+            if lang_id in [2, 28, 51, 57, 61]:
+                return '.c'
+            if lang_id in [3, 29, 52, 58, 62]:
+                return '.cpp'
+            if lang_id in [25, 54]:
+                return '.tar'
+            if lang_id in [66, 67, 101, 102]:
+                return '.S'
+            return None
+
+        form = html.find('form')
+        if form is None:
+            return None
+        lang_list = form.find('select', {'name': 'lang_id'})
+        if lang_list is None:
+            lang_input = form.find('input', {'name': 'lang_id'})
+            if lang_input is not None:
+                return get_suf(lang_input['value'])
+            return None
+        else:
+            langs = [opt['value'] for opt in lang_list.find_all('option') if opt.get('value')]
+            return get_suf(langs[0])
+
+    def statement_available(self):
+        return self._html is not None
 
     def html(self):
-        if self._html is None:
+        if not self.statement_available():
             return 'Statement is not available'
-        # NOTE "kks convert statement.html" will produce a slightly different .md file (more spaces, but rendered markdown is not affected)
-        return self._html.prettify()
+        return str(self._html)
 
     def markdown(self, width=100):
-        if self._html is None:
+        if not self.statement_available():
             return 'Statement is not available'
         converter = HTML2Text(bodywidth=width, baseurl=self.url)
         converter.pad_tables = True
@@ -261,6 +348,13 @@ def get_contest_id(group_id):
     return CONTEST_ID_BY_GROUP.get(group_id, None)
 
 
+def get_group_id(contest_id):
+    for group, contest in CONTEST_ID_BY_GROUP.items():
+        if contest_id == contest:
+            return group
+    return None
+
+
 def get_contest_url(auth_data):
     return f'https://caos.ejudge.ru/ej/client?contest_id={auth_data.contest_id}'
 
@@ -272,17 +366,12 @@ def get_contest_url_with_creds(auth_data):
     return url
 
 
+# NOTE all "ejudge_xxx" methods may raise kks.errors.AuthError
+# If session was used previously to make a request, and no errors were raised, AuthError will not be raised
 def ejudge_summary(session):
     from bs4 import BeautifulSoup
 
-    summary = session.links.get(LinkTypes.SUMMARY, None)
-    if summary is None:
-        return None
-
-    try:
-        page = session.get(summary)
-    except AuthError:
-        return None
+    page = session.get(Links.SUMMARY)
 
     soup = BeautifulSoup(page.content, 'html.parser')
 
@@ -307,14 +396,7 @@ def ejudge_summary(session):
 def ejudge_standings(session):
     from bs4 import BeautifulSoup
 
-    standings = session.links.get(LinkTypes.USER_STANDINGS, None)
-    if standings is None:
-        return None, None
-
-    try:
-        page = session.get(standings)
-    except AuthError:
-        return None, None
+    page = session.get(Links.USER_STANDINGS)
 
     soup = BeautifulSoup(page.content, 'html.parser')
 
@@ -324,7 +406,10 @@ def ejudge_standings(session):
     table = soup.find('table', class_='standings')
     rows = table.find_all('tr')
 
-    task_names = [task.text for task in rows[0].find_all(class_='st_prob')]
+    tasks = [
+        TaskInfo(task.text, extract_contest_name(task.text))
+        for task in rows[0].find_all(class_='st_prob')
+    ]
 
     # skip table header and stats at the bottom
     rows = rows[1:-3]
@@ -338,22 +423,22 @@ def ejudge_standings(session):
                 row.find(class_='st_place').text,
                 user,
                 [
-                    to_task_score(task_name, cell)
-                    for task_name, cell in zip(task_names, score_cells)
+                    to_task_score(task.contest, cell)
+                    for task, cell in zip(tasks, score_cells)
                 ],
                 int(row.find(class_='st_total').text),
                 int(row.find(class_='st_score').text),
                 name == user
             )
 
-    return Standings(task_names, parse_rows())
+    return Standings(tasks, list(parse_rows()))
 
 
 def extract_contest_name(task_name):
     return task_name.split('-')[0]
 
 
-def to_task_score(task_name, cell):
+def to_task_score(contest, cell):
     score = cell.text
     if not score or score.isspace():
         score = None
@@ -365,24 +450,14 @@ def to_task_score(task_name, cell):
         else Status.OK if score is not None \
         else Status.NOT_SUBMITTED
 
-    return TaskScore(extract_contest_name(task_name), task_name, score, status)
-
-
-def ejudge_statement(problem_link, session):
-    # no AuthError
-    page = session.get(problem_link)
-    return Statement(page)
+    return TaskScore(contest, score, status)
 
 
 def ejudge_submissions(session):
     from bs4 import BeautifulSoup
 
-    link = session.links.get(LinkTypes.SUBMISSIONS, None)
-    if link is None:
-        return []
+    page = session.get(Links.SUBMISSIONS, params={'all_runs': 1})
 
-    # no AuthError
-    page = session.get(link, params={'all_runs': 1})
     soup = BeautifulSoup(page.content, 'html.parser')
 
     sub_table = soup.find('table', {'class': 'table'})
@@ -398,7 +473,6 @@ def ejudge_submissions(session):
 def ejudge_report(link, session):
     from bs4 import BeautifulSoup
 
-    # no AuthError
     page = session.get(link)
     soup = BeautifulSoup(page.content, 'html.parser')
     message_table = soup.find('table', {'class': 'message-table'})
@@ -413,10 +487,6 @@ def ejudge_report(link, session):
 def get_problem_info(problem, cache, session):
     from bs4 import BeautifulSoup
 
-    # TODO
-    # - Should max_score be set to 0 after the hard deadline?
-    # - Can there be multiple soft deadlines?
-
     # NOTE penalties are assumed to be the same for all tasks in a contest, it may be wrong
 
     need_loading = False
@@ -436,26 +506,34 @@ def get_problem_info(problem, cache, session):
     if not need_loading:
         return ProblemInfo(full_score, run_penalty, current_penalty)
 
-    def update_cache(full_score, run_penalty, current_penalty, soft_dl):
+    now = datetime.now()
+    def update_cache(full_score, run_penalty, current_penalty, soft_dl, hard_dl):
+        past_deadline = hard_dl is not None and now > hard_dl
+
+        if past_deadline:
+            # see issue #72
+            full_score = 0
+            current_penalty = 0
+
         full_scores[problem.short_name] = full_score
         run_penalties[problem.short_name] = run_penalty
         cache.set('full', full_scores)
         cache.set('run', run_penalties)
 
-        if current_penalty >= full_score and current_penalty != 0:
+        if current_penalty >= full_score and current_penalty != 0 or past_deadline:
             expiration = None
-            # max_score will not change
+            # max_score = 0, will not change
             # NOTE may need patching for kr contests
         else:
             if soft_dl is not None:
                 expiration = soft_dl
             else:
                 expiration = timedelta(days=4, hours=23, minutes=55)
+
         cache.set(penalty_key, current_penalty, expiration)
 
         return ProblemInfo(full_score, run_penalty, current_penalty)
 
-    # no AuthError
     page = session.get(problem.href)
     soup = BeautifulSoup(page.content, 'html.parser')
     task_area = soup.find('div', {'id': 'probNavTaskArea'})
@@ -465,6 +543,7 @@ def get_problem_info(problem, cache, session):
     run_penalty = 0  # may be incorrect for kr
     current_penalty = 0
     soft_dl = None
+    hard_dl = None
 
     if problem_info is None:
         # may happen in kr contests?
@@ -472,7 +551,6 @@ def get_problem_info(problem, cache, session):
 
     # TODO "Full score" can be missing (sm01-3)
     # in this case we should use max score from the table as full
-    # If the penalty can be missing too, we should also parse hard deadlines
     for row in problem_info.find_all('tr'):
         key, value = row.find_all('td')
         if key.text == 'Full score:':
@@ -483,8 +561,10 @@ def get_problem_info(problem, cache, session):
             current_penalty = int(value.text)
         elif key.text == 'Next soft deadline:':
             soft_dl = datetime.strptime(value.text, '%Y/%m/%d %H:%M:%S')
+        elif key.text == 'Deadline:':
+            hard_dl = datetime.strptime(value.text, '%Y/%m/%d %H:%M:%S')
 
-    return update_cache(full_score, run_penalty, current_penalty, soft_dl)
+    return update_cache(full_score, run_penalty, current_penalty, soft_dl, hard_dl)
 
 
 def chunks(iterable, chunk_size):
